Simulation<-function(V,quanti,mod,taille,sigma.quanti=0.18,sigma.quali=0.009){
  ##############
  #AIM: generating a dataset
  ##############
  
  ##############
  #INPUTS:
  #V:variance covariance matrix
  #quanti: labels of continuous variables
  #mod: number of modalities for categirical variables
  #taille: number of individuals
  #sigma.quali, sigma.quanti:variance of niose added on categorical or continuous variables
  ##############
  
  ##############
  #OUTPUTS: a list containing the data set and the data set without noise
  ##############
  equil<-t(apply(matrix(mod,ncol=max(mod),nrow=length(mod)),MARGIN=1,FUN=function(l){c(rep(1/l[1],l[1]),rep(0,length(l)-l[1]))}))
  don<-rmvnorm(taille,sigma=V,method="svd")
  don1<-as.data.frame(don)
  don2<-don1
  quali<- seq(1,ncol(V))[!seq(1,ncol(V))%in%quanti]
  if (length(quanti)!=0){don2[,quanti]<-don2[,quanti]+rmvnorm(taille,mean=rep(0,length(quanti)),sigma=sigma.quanti*diag(length(quanti)))}
  if (length(mod)!=0){don2[,quali]<-don2[,quali]+rmvnorm(taille,mean=rep(0,ncol(V)-length(quanti)),sigma=sigma.quali*diag(ncol(V)-length(quanti)))}
  comp<-1
  for (i in quali){
    don1[,i]<-as.factor(cut(don1[,i],c(-Inf,quantile(don1[,i],cumsum(equil[comp,,drop=F])[-c(mod[comp]:max(mod))]),Inf)))
    don2[,i]<-as.factor(cut(don2[,i],c(-Inf,quantile(don2[,i],cumsum(equil[comp,,drop=F])[-c(mod[comp]:max(mod))]),Inf)))
    comp<-comp+1
    levels(don1[,i])<-paste("v",i,"_",1:nlevels(don1[,i]),sep="")
    levels(don2[,i])<-paste("v",i,"_",1:nlevels(don2[,i]),sep="")
  }
  return(list(Jeu_non_bruite=don1,Jeu_bruite=don2))
}

SNR<-function(Jeu.nb,Jeu){
  ##############
  #AIM: estimating the signal to noise ratio (snr)
  ##############
  
  ##############
  #INPUTS:
  #Jeu.nb: a data set without noise
  #Jeu: a data set with noise
  ##############
  
  ##############
  #OUTPUT:
  #a list containing the snr estimated for continuous and categorical variables
  ##############
  est.snr.quanti<-c()
  est.snr.quali<-c()
  if(sum(MonvarClass(Jeu.nb)=="numeric")!=0){
    eig<-PCA(Jeu.nb[,which(MonvarClass(Jeu.nb)=="numeric")],ncp=length(MonvarClass(Jeu.nb)=="numeric"),graph=F)$eig
    var.sign<-PCA(Jeu[,which(MonvarClass(Jeu.nb)=="numeric")],ncp=sum(eig[,3]<100)+1,graph=F)$eig[sum(round(eig[,3])<100)+1,3]
    est.snr.quanti<-sqrt(var.sign/(100-var.sign))
  }
  if(sum(MonvarClass(Jeu.nb)=="factor")!=0){  
    eig<-MCA(Jeu.nb[,which(MonvarClass(Jeu.nb)=="factor")],ncp=length(MonvarClass(Jeu.nb)=="factor"),graph=F)$eig
    var.sign<-MCA(Jeu[,which(MonvarClass(Jeu.nb)=="factor")],ncp=sum(eig[,3]<100)+1,graph=F)$eig[sum(round(eig[,3])<100)+1,3]
    est.snr.quali<-sqrt(var.sign/(100-var.sign))
    est.snr<-list(est.snr.quanti,est.snr.quali)
  }
  est.snr<-list(est.snr.quanti,est.snr.quali)
  names(est.snr)<-c("snr quanti","snr quali")
  return(est.snr)
}

Simulation.snr<-function(V,quanti,mod,taille,snr.quanti=2,snr.quali=2,seed=sample(1:100000000,size=1)){
  ##############
  #AIM: Simulating a data set specifying the signal to noise ratio
  ##############
  
  ##############
  #INPUTS:
  #V:variance covariance matrix
  #quanti: labels of continuous variables
  #mod: number of modalities for categirical variables
  #taille: number of individuals
  #snr.quanti/snr.quali: signal to noise ratio concerning continuous or categorical variables
  #seed: the seed for the random generator
  ##############
  
  ##############
  #OUTPUTS: a list containing the data set and the data set without noise, as well as the corresponding variability of the noise
  ##############
  sigma_quanti<-NA
  if(length(quanti)!=0){
    sigma_quanti<-diag(V[quanti,quanti])/(snr.quanti^2*length(quanti))
  }
  sigma_quali<-NA
  if (length(mod)>0){
    if(snr.quali<1){warning("Attention, snr quali faible")}
    Sigma.quali<-c(seq(0.0001,0.01,0.0001),seq(0,1.5,0.01))
    tol.quali<-0.1
    for (i in 1:length(Sigma.quali)){
      set.seed(seed)
      J<-Simulation(V=V,quanti,mod,taille=taille,sigma.quanti=sigma_quanti,sigma.quali=Sigma.quali[i])
      Jeu<-J[[2]]
      Jeu.nb<-J[[1]]
      sign<-SNR(Jeu.nb,Jeu)[[2]]
      sigma_quali<-Sigma.quali[i]
      if ((sign<=(snr.quali+tol.quali)) & (sign>=(snr.quali-tol.quali))){break}
      sigma_quali<-NA
    }
  }
  if ((is.na(sigma_quali)&is.na(sigma_quanti))||(is.na(sigma_quali)&((length(mod)>0)))|| (is.na(sigma_quanti) & (length(quanti)>0))){J<-list(c(),c())}else(J<-list(Jeu.nb,Jeu))
  names(J)<-c("Jeu non bruite","Jeu bruite")
  sigma<-list(sigma_quanti,sigma_quali)
  names(sigma)<-c("sigma.quanti","sigma.quali")
  return(list(J=J,sigma=sigma))
}

Mcar<-function(don,p=0.1){
  ##############
  #AIM: generating missing values from a dataframe
  ##############
  
  ##############
  #INPUTS:
  #don: a dataframe
  #p: the proportion of missing values
  ##############
  
  ##############
  #OUPUTS: a dataframe with missing values
  ##############
  x<-don
  n <- nrow(don)
  m <- ncol(don)
  NAloc <- rep(FALSE,n*m)
  NAloc[sample(n * m, floor(n * m * p))] <- TRUE
  x[matrix(NAloc, nrow = n, ncol = m)] <- NA
  return(as.data.frame(x))
}


nrmse<-function(x.imp,x.na,x.true){
  ##############
  #AIM: calculating the NRMSE
  ##############

  ##############
  #INPUTS:
  #x.imp: an imputed data set
  #x.na: a data set with missing values
  #x.true: a data set containg true values
  ##############
  
  ##############
  #OUTPUTS:
  #the corresponding NRMSE
  ##############
  mis <- is.na(x.na)
  temp1<-matrix(0,ncol=ncol(x.imp),nrow=nrow(x.true))
  temp1[mis]<-x.imp[mis]
  temp2<-matrix(0,ncol=ncol(x.imp),nrow=nrow(x.true))
  temp2[mis]<-x.true[mis]
  return(sqrt((1/sum(apply(mis,MARGIN=2,FUN=any)))*sum(colSums((temp1 - temp2)[,apply(mis,MARGIN=2,FUN=any),drop=F]^{2})/colSums(mis[,apply(mis,MARGIN=2,FUN=any),drop=F])/(((nrow(x.true)-1)/nrow(x.true))*diag(var(x.true[,apply(mis,MARGIN=2,FUN=any),drop=F]))))))
}

MonmixError<-function (ximp, xmis, xtrue){
  ##############
  #AIM: calculating the NRMSE and the PFC
  ##############
  
  ##############
  #INPUTS:
  #ximp: an imputed data set
  #xmis: a data set with missing values
  #xtrue: a data set containg true values
  ##############
  
  ##############
  #OUTPUTS:
  #a matrix containing the NRMSE and the PFC
  ##############
    PFC<-NA
    NRMSE<-NA
    if (class(ximp) == "MonmissForest") 
        stop("'xmis' is not of class 'MonmissForest' - maybe you forgot to point at the\n  list element $ximp from the MonmissForest output object.")
    x.types <- MonvarClass(ximp)
    n <- nrow(ximp)
    k <- length(unique(x.types))
    for (t.type in x.types) {
        t.ind <- which(x.types == t.type)
        if (t.type == "numeric") {
	      NRMSE<-nrmse(ximp[, t.ind,drop=F], xmis[, t.ind,drop=F], xtrue[, 
			      t.ind,drop=F])
        }
        else {
	  temp1<-matrix(0,ncol=ncol(ximp),nrow=n)
	  temp1[is.na(xmis)]<-ximp[is.na(xmis)]
	  temp2<-matrix(0,ncol=ncol(ximp),nrow=n)
	  temp2[is.na(xmis)]<-xtrue[is.na(xmis)]
	  dist <- sum(as.character(as.matrix(temp1[,x.types == "factor"])) != 
	      as.character(as.matrix(temp2[,x.types == "factor"])))
	  no.na <- sum(is.na(xmis[, x.types == "factor"]))
	    if (no.na == 0) {
		PFC <- 0
	    }
	    else {
		PFC <- dist/no.na
	    }
        }
    }
    res<-matrix(c(PFC,NRMSE),ncol=1,nrow=2)
    rownames(res)<-c("PFC","NRMSE")
    return(res)
}

AFDM.RF<-function(l,don.na,ncp=2,RF=TRUE){
  ##############
  #AIM: performing imputation using the missForest algorithm and the FAMD algorithm
  #and calculating the error for each method
  ##############
  
  ##############
  #INPUTS:
  #l: a list containing an data set with and without noise (output from the "Simulation" function)
  #don.na: a data set with missing values
  #ncp: the number of principal components used for the FAMD algorithm
  #RF: a boolean indicating if the random forest algorithm is performed
  ##############
  
  ##############
  #OUTPUTS:
  #NRMSE and PFC for each methods
  ##############
  rename<-function(completeobs,l){
    res<-completeobs
    for (i in 1:ncol(res)){
      if(any(!(levels(res[,i])==levels(l[[1]][,i])))){
	levels(res[,i])<-gsub(paste(colnames(res)[i],"_",sep=""),"",levels(res[,i]))
      }
    }
  return(res)
  }
  don.imp<-rename(imputeFAMD(don.na,method="Regularized",ncp=ncp)$compl,l)
  res.famd<-cbind(MonmixError(ximp=don.imp,xmis=don.na,xtrue=l[[1]]),MonmixError(ximp=don.imp,xmis=don.na,xtrue=l[[2]]))
  colnames(res.famd)<-c("Jeu non bruite","Jeu bruite")
  if (RF){
    don.imp.rf<-MonmissForest(don.na)$ximp
    res.rf<-cbind(MonmixError(ximp=don.imp.rf,xmis=don.na,xtrue=l[[1]]),MonmixError(ximp=don.imp.rf,xmis=don.na,xtrue=l[[2]]))
  }else{
    res.rf<-matrix(NA,ncol=ncol(res.famd),nrow=nrow(res.famd),dimnames=list(c("PFC","NRMSE"),c("Jeu non bruite","Jeu bruite")))
    don.imp.rf<-NA
  }
  res<-list(don.na,c(),c(),res.famd,res.rf)
  names(res)<-c("NA","ImpAFDM","ImpRF","ErrAFDM","ErrRF")
  return(res)
}

Comparaison<-function(Jeu,Jeu.nb=NULL,borne=10,Sim=200,name.jeu=c("Nouveau_jeu"),RF=T){
  ##############
  #AIM: performing simulations adding missing values several times on a same data set
  ##############
  
  ##############
  #INPUTS:
  #Jeu: a noisy data set
  #Jeu.nb: a data set without noise
  #borne: the maximum number of components used for the FAMD algorithm
  #Sim: the number of simulations (correspondind to the number of patterns of missing values
  #for predifined proportion of missing values)
  #names.jeu: a name for the data set
  #RF: a boolean indicating if the random forest algorithm is performed
  ##############
  
  ##############
  #OUTPUTS:
  #A list containing the errors for the several configurations
  ##############
  Taux<-c(0.1,0.2,0.3)
  names.p<-c("10%","20%","30%")
  names.sim<-c()
  for (i in 1:Sim){names.sim<-c(names.sim,paste("Simulation ",i,sep=""))}
  names.ncp<-c()
  for (i in 1:borne){names.ncp<-c(names.ncp,paste(i," Composantes",sep=""))}
  Res.jeu<-list()
  Res.sim<-list()
  for (sim in 1:Sim){
  print(paste(name.jeu,sim,sep=" "))
    Res.p<-list()
    for (p in Taux){
      Jeu.na<-Mcar(Jeu,p=p)
      Res.ncp<-list()
	for (ncp in 1:borne){
	  if (is.null(Jeu.nb)){
	    Res.ncp<-c(Res.ncp,try(list(AFDM.RF(list(Jeu,Jeu),ncp=ncp,don.na=Jeu.na,RF=(ncp==1&RF)))))
	  }else{
	    Res.ncp<-c(Res.ncp,try(list(AFDM.RF(list(Jeu.nb,Jeu),ncp=ncp,don.na=Jeu.na,RF=(ncp==1&RF)))))}
	}
      names(Res.ncp)<-names.ncp[1:length(Res.ncp)]
      Res.p<-c(Res.p,list(Res.ncp))
    }
    names(Res.p)<-names.p[1:length(Res.p)]
    Res.sim<-c(Res.sim,list(Res.p))
  }
  names(Res.sim)<-names.sim
  Res.jeu<-list(Res.sim)
  names(Res.jeu)<-name.jeu
  return(Res.jeu)
}

echelle<-function(err){return(c(0,max(1,max(err))))}
#useful function for graphics

Err.nbaxe<-function(Res.Jeu){
  ##############
  #AIM: Plotting the error according to the number of dimensions
  ##############
  
  ##############
  #INPUT:
  #Res.jeu: an output from the function "COmparaison"
  ##############
  
  ##############
  #OUTPUT:
  #two graphics representing the error according to the number of dimensions
  ##############
  
  type<-3
  Sim<-length(Res.Jeu[[1]])
  Taux<-1:3
  Indice.type<-list(list(c(2),c(1),c(1,2)),list(c(4),c(3),c(3,4)),list(c(6),c(5),c(5,6)))
  f<-function(x){if (is.na(x[1])){return(NA)}else{return(as.matrix(which.max(x)))}}
  for (jeu in 1:length(Res.Jeu)){
    Err.axe<-matrix(0,ncol=length(Res.Jeu[[1]][[1]][[1]]),nrow=6)
    div<-list(rep(0,length(Res.Jeu[[1]][[1]][[1]])),rep(0,length(Res.Jeu[[1]][[1]][[1]])),rep(0,length(Res.Jeu[[1]][[1]][[1]])))
    names(div)<-c("10%","20%","30%")
    for (sim in 1:Sim){
      Temp.taux<-c()
      for (taux in Taux){
	    Temp<-c()
	    for (i in 1:length(Res.Jeu[[1]][[sim]][[taux]])){
	      Temp<-cbind(Temp,
		  if(mode(Res.Jeu[[1]][[sim]][[taux]][[i]])=="list"){
		  Res.Jeu[[1]][[sim]][[taux]][[i]][["ErrAFDM"]][,1]
		  }else{matrix(NA,ncol=1,nrow=6)})
	    }
	    Temp.taux<-rbind(Temp.taux,Temp)
      }
      Temp.taux.axe<-matrix(0,ncol=length(Res.Jeu[[1]][[1]][[1]]),nrow=6)
      div[[1]]<-rep(1,ncol(Temp.taux[Indice.type[[1]][[type]],,drop=F]))-as.numeric(apply((is.na(Temp.taux[Indice.type[[1]][[type]],,drop=F])),MARGIN=2,FUN="any"))+div[[1]]
      div[[2]]<-rep(1,ncol(Temp.taux[Indice.type[[2]][[type]],,drop=F]))-as.numeric(apply((is.na(Temp.taux[Indice.type[[2]][[type]],,drop=F])),MARGIN=2,FUN="any"))+div[[2]]
      div[[3]]<-rep(1,ncol(Temp.taux[Indice.type[[3]][[type]],,drop=F]))-as.numeric(apply((is.na(Temp.taux[Indice.type[[3]][[type]],,drop=F])),MARGIN=2,FUN="any"))+div[[3]]
      Temp.taux.axe[!is.na(Temp.taux)]<-Temp.taux[!is.na(Temp.taux)]
      Err.axe<-Err.axe+Temp.taux.axe
    }		
  }
  #Analyse
  Evol.err.axe<-function(Err.axe,div){
    Mean.err.axe<-Err.axe
    Mean.err.axe[1:2,]<-sweep(Err.axe[1:2,],MARGIN=2,div[[1]],FUN="/")
    Mean.err.axe[3:4,]<-sweep(Err.axe[3:4,],MARGIN=2,div[[2]],FUN="/")
    Mean.err.axe[5:6,]<-sweep(Err.axe[5:6,],MARGIN=2,div[[3]],FUN="/")
    col<-c(gray(.1),gray(.4),gray(.7))
    par(mfrow=c(2,1))
    matplot(t(Mean.err.axe[c(1,3,5),]),type=c("l","l","l"),ylab="PFC",xlab="Nb of dimensions",main="Error according to the number of dimensions",lty = c("solid", "solid"),col=col)
    legend("topright",legend=c("10%","20%","30%"),col=col,lty=c("solid", "solid"))
    matplot(t(Mean.err.axe[c(2,4,6),]),type=c("l","l","l"),ylab="NRMSE",xlab="Nb of dimensions",main="Error according to the number of dimensions",lty = c("solid", "solid"),col=col)
    legend("topright",legend=c("10%","20%","30%"),col=col,lty=c("solid", "solid"))
  }
  Evol.err.axe(Err.axe,div)
}

Erreur<-function(Comp,borne=length(Comp[[1]][[1]][["10%"]])){
  ##############
  #AIM: transforming a output from the function Comparaison as a list of matrix
  ##############
  
  ##############
  #INPUTS:
  #Comp: a output from the function "Comparaison"
  #borne: the maximum number of principal components considired
  ##############
  
  ##############
  #OUTPUTS:
  #a list of matrix containing erros for the random forest algorithm and FAMD algorithm
  ##############
  
  type<-3
  if (ncol(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])==sum(MonvarClass(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])=="numeric")){
    type<-1
  }else if(ncol(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])==sum(MonvarClass(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])=="factor")){
    type<-2
  }
  crit<-list(c("NRMSE"),c("PFC"),c("PFC","NRMSE"))
  Sim<-length(Comp[[1]])
  if(type!=3){
    err.afdm<-array(dim=c(borne,3,Sim))
    err.rf<-array(dim=c(3,Sim))
    for(nbaxe in 1:borne){
      for (i in 1:Sim){
	for (taux in 1:3){
	  err.afdm[nbaxe,taux,i]<-Comp[[1]][[i]][[taux]][[nbaxe]][["ErrAFDM"]][crit[[type]],1]
	  err.rf[taux,i]<-Comp[[1]][[i]][[taux]][[1]][["ErrRF"]][crit[[type]],1]
	}
      }
    }
  }else{
    err.afdm<-list(NRMSE=array(dim=c(borne,3,Sim),dimnames=list(paste("ncp =",1:borne),c("10%","20%","30%"),paste("sim",1:Sim))),
		    PFC=array(dim=c(borne,3,Sim),dimnames=list(paste("ncp =",1:borne),c("10%","20%","30%"),paste("sim",1:Sim))))
    err.rf<-list(NRMSE=array(dim=c(3,Sim),dimnames=list(c("10%","20%","30%"),paste("sim",1:Sim))),
		    PFC=array(dim=c(3,Sim),dimnames=list(c("10%","20%","30%"),paste("sim",1:Sim))))
    for(nbaxe in 1:borne){
	for (i in 1:Sim){
	  for (taux in 1:3){
	    err.afdm[[1]][nbaxe,taux,i]<-Comp[[1]][[i]][[taux]][[nbaxe]][["ErrAFDM"]][crit[[1]],1]
	    err.afdm[[2]][nbaxe,taux,i]<-Comp[[1]][[i]][[taux]][[nbaxe]][["ErrAFDM"]][crit[[2]],1]
	    err.rf[[1]][taux,i]<-Comp[[1]][[i]][[taux]][[1]][["ErrRF"]][crit[[1]],1]
	    err.rf[[2]][taux,i]<-Comp[[1]][[i]][[taux]][[1]][["ErrRF"]][crit[[2]],1]
	  }
	}
      }
  }
  err<-list(err.afdm=err.afdm,err.rf=err.rf)
  return(err)
}

Analyse<-function(Comp,borne=length(Comp[[1]][[1]][["10%"]]),ncp=NULL,crit.aff=NULL){
  ##############
  #AIM: Plotting boxplots of error from the output of function "Comparaison"
  ##############
  
  ##############
  #INPUTS: 
  #Comp: a output from the function Comparaison
  #borne: the maximum number of principal components consedired
  #ncp: a number to focus on a particular number of prnicipal components
  #crit.aff: a number indicating the criterion to plot (1 for NRMSE, 2 for PFC, 3 for both)
  ##############
  
  ##############
  #OUTPUTS:
  #boxplots of errors for each percentage of missing values and each methods,
  #for the number of principal components considered
  ##############
  
  if (borne==1){ncp<-1}
    res<-Erreur(Comp,borne=borne)
    if (ncol(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])==sum(MonvarClass(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])=="numeric")){
      type<-1
    }else if(ncol(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])==sum(MonvarClass(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])=="factor")){
      type<-2
    }else{
      type<-3
  }
  if (is.null(crit.aff)){crit.aff<-type}
  err.afdm<-res[[1]]
  err.rf<-res[[2]]
  if (is.null(ncp)){
      if (type==3){
	  par(mfrow=c(2,3),mar = c(2.1, 4.1, 4.1, 2.1))
	  boxplot(data.frame(err.rf[[1]][1,],t(err.afdm[[1]][,1,])),col=c("white",rep(gray(.6),dim(err.afdm[[1]])[[1]])),ylab = "NRMSE",main=paste(names(Comp)," 10%",sep=""),ylim=c(min(err.rf[[1]][1,]),min(max(err.afdm[[1]][,1,]),1.1)))
	  boxplot(data.frame(err.rf[[1]][2,],t(err.afdm[[1]][,2,])),col=c("white",rep(gray(.6),dim(err.afdm[[1]])[[1]])),ylab = "NRMSE",main=paste(names(Comp)," 20%",sep=""),ylim=c(min(err.rf[[1]][2,]),min(max(err.afdm[[1]][,2,]),1.1)))
	  boxplot(data.frame(err.rf[[1]][3,],t(err.afdm[[1]][,3,])),col=c("white",rep(gray(.6),dim(err.afdm[[1]])[[1]])),ylab = "NRMSE",main=paste(names(Comp)," 30%",sep=""),ylim=c(min(err.rf[[1]][3,]),min(max(err.afdm[[1]][,3,]),1.1)))
	  boxplot(data.frame(err.rf[[2]][1,],t(err.afdm[[2]][,1,])),col=c("white",rep(gray(.6),dim(err.afdm[[1]])[[1]])),ylab = "PFC",main=paste(names(Comp)," 10%",sep=""))
	  boxplot(data.frame(err.rf[[2]][2,],t(err.afdm[[2]][,2,])),col=c("white",rep(gray(.6),dim(err.afdm[[1]])[[1]])),ylab = "PFC",main=paste(names(Comp)," 20%",sep=""))
	  boxplot(data.frame(err.rf[[2]][3,],t(err.afdm[[2]][,3,])),col=c("white",rep(gray(.6),dim(err.afdm[[1]])[[1]])),ylab = "PFC",main=paste(names(Comp)," 30%",sep=""))
	}else if(type==1){
	  par(mfrow=c(1,3),mar = c(2.1, 4.1, 4.1, 2.1))
	  boxplot(data.frame(err.rf[1,],t(err.afdm[,1,])),col=c("white",rep(gray(.6),dim(err.afdm)[[1]])),ylab = "NRMSE",main=paste(names(Comp)," 10%",sep=""),ylim=c(min(err.rf[1,]),min(max(err.afdm[,1,]),1.1)))
	  boxplot(data.frame(err.rf[2,],t(err.afdm[,2,])),col=c("white",rep(gray(.6),dim(err.afdm)[[1]])),ylab = "NRMSE",main=paste(names(Comp)," 20%",sep=""),ylim=c(min(err.rf[2,]),min(max(err.afdm[,2,]),1.1)))
	  boxplot(data.frame(err.rf[3,],t(err.afdm[,3,])),col=c("white",rep(gray(.6),dim(err.afdm)[[1]])),ylab = "NRMSE",main=paste(names(Comp)," 30%",sep=""),ylim=c(min(err.rf[3,]),min(max(err.afdm[,3,]),1.1)))
	}else if(type==2){
	  par(mfrow=c(1,3),mar = c(2.1, 4.1, 4.1, 2.1))
	  boxplot(data.frame(err.rf[1,],t(err.afdm[,1,])),col=c("white",rep(gray(.6),dim(err.afdm)[[1]])),ylab = "PFC",main=paste(names(Comp)," 10%",sep=""))
	  boxplot(data.frame(err.rf[2,],t(err.afdm[,2,])),col=c("white",rep(gray(.6),dim(err.afdm)[[1]])),ylab = "PFC",main=paste(names(Comp)," 20%",sep=""))
	  boxplot(data.frame(err.rf[3,],t(err.afdm[,3,])),col=c("white",rep(gray(.6),dim(err.afdm)[[1]])),ylab = "PFC",main=paste(names(Comp)," 30%",sep=""))
	}
  }else{
      if (type==3){
	if (crit.aff==3){par(mfrow=c(2,1),mar = c(2.1, 4.1, 4.1, 2.1))}else{par(mar = c(2.1, 4.1, .8, 2.1))}
	if(crit.aff!=2){
	  boxplot(data.frame(err.rf[[1]][1,],err.afdm[[1]][ncp,1,],err.rf[[1]][2,],err.afdm[[1]][ncp,2,],err.rf[[1]][3,],err.afdm[[1]][ncp,3,]),col=rep(c("white",gray(.6)),3),ylab = "NRMSE",main=names(Comp),names=c("RF 10%","FAMD 10%","RF 20%","FAMD 20%","RF 30%","FAMD 30%"),
		  ylim=echelle(data.frame(err.rf[[1]][1,],err.afdm[[1]][ncp,1,],err.rf[[1]][2,],err.afdm[[1]][ncp,2,],err.rf[[1]][3,],err.afdm[[1]][ncp,3,])))}
	if(crit.aff!=1){
	  boxplot(data.frame(err.rf[[2]][1,],err.afdm[[2]][ncp,1,],err.rf[[2]][2,],err.afdm[[2]][ncp,2,],err.rf[[2]][3,],err.afdm[[2]][ncp,3,]),col=rep(c("white",gray(.6)),3),ylab = "PFC",names=c("RF 10%","FAMD 10%","RF 20%","FAMD 20%","RF 30%","FAMD 30%")
		  ,ylim=echelle(data.frame(err.rf[[2]][1,],err.afdm[[2]][ncp,1,],err.rf[[2]][2,],err.afdm[[2]][ncp,2,],err.rf[[2]][3,],err.afdm[[2]][ncp,3,])))}
      }else if(type==1){
	  par(mar = c(2.1, 4.1, .8, 2.1))
	  boxplot(data.frame(err.rf[1,],err.afdm[ncp,1,],err.rf[2,],err.afdm[ncp,2,],err.rf[3,],err.afdm[ncp,3,]),col=rep(c("white",gray(.6)),3),ylab = "NRMSE",main=names(Comp),names=c("RF 10%","FAMD 10%","RF 20%","FAMD 20%","RF 30%","FAMD 30%"),
		  ylim=echelle(data.frame(err.rf[1,],err.afdm[ncp,1,],err.rf[2,],err.afdm[ncp,2,],err.rf[3,],err.afdm[ncp,3,])))
      }else if(type==2){
	  par(mar = c(2.1, 4.1, .8, 2.1))
	  boxplot(data.frame(err.rf[1,],err.afdm[ncp,1,],err.rf[2,],err.afdm[ncp,2,],err.rf[3,],err.afdm[ncp,3,]),col=rep(c("white",gray(.6)),3),ylab = "PFC",main=names(Comp),names=c("RF 10%","FAMD 10%","RF 20%","FAMD 20%","RF 30%","FAMD 30%")
		  ,ylim=echelle(data.frame(err.rf[1,],err.afdm[ncp,1,],err.rf[2,],err.afdm[ncp,2,],err.rf[3,],err.afdm[ncp,3,])))
	  }
  }
}


Analyse3.1<-function(Comp,borne=length(Comp[[1]][[1]][["10%"]]),ncp=NULL){
  ##############
  #AIM: transforming an output from the function Comparaison in order to plot 
  #figures of section 3.1
  ##############
  
  ##############
  #INPUTS: 
  #Comp: a output from the function Comparaison
  #borne: the maximum number of principal components consedired
  #ncp: a number to specify a particular number of principal components
  ##############
  
  ##############
  #OUTPUTS:
  #errors to draw for each percentage of missing values
  #for the number of principal components considered
  ##############
  
  if (borne==1){ncp<-1}
  res<-Erreur(Comp,borne=borne)
  if (ncol(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])==sum(MonvarClass(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])=="numeric")){
      type<-1
  }else if(ncol(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])==sum(MonvarClass(Comp[[1]][[1]][["10%"]][["1 Composantes"]][["NA"]])=="factor")){
      type<-2
  }else{
      type<-3
  }
  err.afdm<-res[[1]]
  err.rf<-res[[2]]
  if (type==3){
      val<-list(quanti=data.frame(err.afdm[[1]][ncp,1,],err.afdm[[1]][ncp,2,],err.afdm[[1]][ncp,3,]),quali=data.frame(err.afdm[[2]][ncp,1,],err.afdm[[2]][ncp,2,],err.afdm[[2]][ncp,3,]))
      return(val)
  }else{
      val<-list(data.frame(err.afdm[ncp,1,],err.afdm[ncp,2,],err.afdm[ncp,3,]))
      return(val)
  }
}